@@ -45,7 +45,7 @@ class Invalid(Exception):
     argument, defaulting to ``None``.  The ``msg`` argument is a
     freeform field indicating the error circumstance.
 
-    The constructor additionally may receive a an optional ``value``
+    The constructor additionally may receive an optional ``value``
     keyword, indicating the value related to the error.
     """
     pos = None
@@ -223,11 +223,11 @@ class Regex(object):
 
         The ``regex`` argument may also be a pattern object (the
         result of ``re.compile``) instead of a string.
-        
-        When calling, if ``value`` matches the regular expression, 
-        validation succeeds; otherwise, :exc:`colander.Invalid` is 
-        raised with the ``msg`` error message. 
-    """    
+
+        When calling, if ``value`` matches the regular expression,
+        validation succeeds; otherwise, :exc:`colander.Invalid` is
+        raised with the ``msg`` error message.
+    """
     def __init__(self, regex, msg=None):
         if isinstance(regex, basestring):
             self.match_object = re.compile(regex)
@@ -243,10 +243,10 @@ class Regex(object):
             raise Invalid(node, self.msg)
 
 class Email(Regex):
-    """ Email address validator. If ``msg`` is supplied, it will be 
-        the error message to be used when raising :exc:`colander.Invalid`; 
-        otherwise, defaults to 'Invalid email address'.  
-    """    
+    """ Email address validator. If ``msg`` is supplied, it will be
+        the error message to be used when raising :exc:`colander.Invalid`;
+        otherwise, defaults to 'Invalid email address'.
+    """
     def __init__(self, msg=None):
         if msg is None:
             msg = _("Invalid email address")
@@ -336,12 +336,26 @@ class OneOf(object):
 
 class SchemaType(object):
     """ Base class for all schema types """
-    def flatten(self, node, appstruct, prefix=''):
+    def flatten(self, node, appstruct, prefix='', listitem=False):
         result = {}
-        selfname = '%s%s' % (prefix, node.name)
+        if listitem:
+            selfname = prefix
+        else:
+            selfname = '%s%s' % (prefix, node.name)
         result[selfname] = appstruct
         return result
 
+    def unflatten(self, node, paths, fstruct):
+        name = node.name
+        assert paths == [name], "paths should be [name] for leaf nodes."
+        return fstruct[name]
+
+    def set_value(self, node, appstruct, path, value):
+        raise AssertionError("Can't call 'set_value' on a leaf node.")
+
+    def get_value(self, node, appstruct, path):
+        raise AssertionError("Can't call 'set_value' on a leaf node.")
+
 class Mapping(SchemaType):
     """ A type which represents a mapping of names to nodes.
 
@@ -441,9 +455,9 @@ class Mapping(SchemaType):
 
         if error is not None:
             raise error
-                
+
         return result
-        
+
     def serialize(self, node, appstruct):
         if appstruct is null:
             appstruct = {}
@@ -462,19 +476,42 @@ class Mapping(SchemaType):
 
         return self._impl(node, cstruct, callback)
 
-    def flatten(self, node, appstruct, prefix=''):
+    def flatten(self, node, appstruct, prefix='', listitem=False):
         result = {}
-        selfname = '%s%s' % (prefix, node.name)
-        selfprefix = selfname + '.'
-        result[selfname] = appstruct
+        if listitem:
+            selfprefix = prefix
+        else:
+            selfprefix = '%s%s.' % (prefix, node.name)
 
-        for num, subnode in enumerate(node.children):
+        for subnode in node.children:
             name = subnode.name
             substruct = appstruct.get(name, null)
             result.update(subnode.typ.flatten(subnode, substruct,
                                               prefix=selfprefix))
         return result
-        
+
+    def unflatten(self, node, paths, fstruct):
+        return _unflatten_mapping(node, paths, fstruct)
+
+    def set_value(self, node, appstruct, path, value):
+        if '.' in path:
+            next_name, rest = path.split('.', 1)
+            next_node = node[next_name]
+            next_appstruct = appstruct[next_name]
+            appstruct[next_name] = next_node.typ.set_value(
+                next_node, next_appstruct, rest, value)
+        else:
+            appstruct[path] = value
+        return appstruct
+
+    def get_value(self, node, appstruct, path):
+        if '.' in path:
+            name, rest = path.split('.', 1)
+            next_node = node[name]
+            return next_node.typ.get_value(next_node, appstruct[name], rest)
+        return appstruct[path]
+
+
 class Positional(object):
     """
     Marker abstract base class meaning 'this type has children which
@@ -530,7 +567,7 @@ class Tuple(Positional, SchemaType):
                 if error is None:
                     error = Invalid(node)
                 error.add(e, num)
-                
+
         if error is not None:
             raise error
 
@@ -548,17 +585,18 @@ class Tuple(Positional, SchemaType):
     def deserialize(self, node, cstruct):
         if cstruct is null:
             return null
-        
+
         def callback(subnode, subval):
             return subnode.deserialize(subval)
 
         return self._impl(node, cstruct, callback)
 
-    def flatten(self, node, appstruct, prefix=''):
+    def flatten(self, node, appstruct, prefix='', listitem=False):
         result = {}
-        selfname = '%s%s' % (prefix, node.name)
-        selfprefix = selfname + '.'
-        result[selfname] = appstruct
+        if listitem:
+            selfprefix = prefix
+        else:
+            selfprefix = '%s%s.' % (prefix, node.name)
 
         for num, subnode in enumerate(node.children):
             substruct = appstruct[num]
@@ -566,6 +604,47 @@ class Tuple(Positional, SchemaType):
                                               prefix=selfprefix))
         return result
 
+    def unflatten(self, node, paths, fstruct):
+        mapstruct = _unflatten_mapping(node, paths, fstruct)
+        appstruct = []
+        for subnode in node.children:
+            appstruct.append(mapstruct[subnode.name])
+        return tuple(appstruct)
+
+    def set_value(self, node, appstruct, path, value):
+        appstruct = list(appstruct)
+        if '.' in path:
+            next_name, rest = path.split('.', 1)
+        else:
+            next_name, rest = path, None
+        for index, next_node in enumerate(node.children):
+            if next_node.name == next_name:
+                break
+        else:
+            raise KeyError(next_name)
+        if rest is not None:
+            next_appstruct = appstruct[index]
+            appstruct[index] = next_node.typ.set_value(
+                next_node, next_appstruct, rest, value)
+        else:
+            appstruct[index] = value
+        return tuple(appstruct)
+
+    def get_value(self, node, appstruct, path):
+        if '.' in path:
+            name, rest = path.split('.', 1)
+        else:
+            name, rest = path, None
+        for index, next_node in enumerate(node.children):
+            if next_node.name == name:
+                break
+        else:
+            raise KeyError(name)
+        if rest is not None:
+            return next_node.typ.get_value(next_node, appstruct[index], rest)
+        return appstruct[index]
+
+
 class Sequence(Positional, SchemaType):
     """
     A type which represents a variable-length sequence of nodes,
@@ -623,7 +702,7 @@ class Sequence(Positional, SchemaType):
                 if error is None:
                     error = Invalid(node)
                 error.add(e, num)
-                
+
         if error is not None:
             raise error
 
@@ -679,29 +758,64 @@ class Sequence(Positional, SchemaType):
         """
         if cstruct is null:
             return null
-        
+
         def callback(subnode, subcstruct):
             return subnode.deserialize(subcstruct)
 
         return self._impl(node, cstruct, callback, accept_scalar)
 
-    def flatten(self, node, appstruct, prefix=''):
+    def flatten(self, node, appstruct, prefix='', listitem=False):
         result = {}
-        selfname = '%s%s' % (prefix, node.name)
-        selfprefix = selfname + '.'
-        result[selfname] = appstruct
+        if listitem:
+            selfprefix = prefix
+        else:
+            selfprefix = '%s%s.' % (prefix, node.name)
 
         childnode = node.children[0]
 
         for num, subval in enumerate(appstruct):
             subname = '%s%s' % (selfprefix, num)
-            result[subname] = subval
             subprefix = subname + '.'
-            result.update(childnode.typ.flatten(childnode, subval,
-                                                prefix=subprefix))
+            result.update(childnode.typ.flatten(
+                childnode, subval, prefix=subprefix, listitem=True))
 
         return result
 
+    def unflatten(self, node, paths, fstruct):
+        only_child = node.children[0]
+        child_name = only_child.name
+        def get_child(name):
+            return only_child
+        def rewrite_subpath(subpath):
+            if '.' in subpath:
+                suffix = subpath.split('.', 1)[1]
+                return '%s.%s' % (child_name, suffix)
+            return child_name
+        mapstruct = _unflatten_mapping(node, paths, fstruct,
+                                       get_child, rewrite_subpath)
+        return [mapstruct[str(index)] for index in xrange(len(mapstruct))]
+
+    def set_value(self, node, appstruct, path, value):
+        if '.' in path:
+            next_name, rest = path.split('.', 1)
+            index = int(next_name)
+            next_node = node.children[0]
+            next_appstruct = appstruct[index]
+            appstruct[index] = next_node.typ.set_value(
+                next_node, next_appstruct, rest, value)
+        else:
+            index = int(path)
+            appstruct[index] = value
+        return appstruct
+
+    def get_value(self, node, appstruct, path):
+        if '.' in path:
+            name, rest = path.split('.', 1)
+            index = int(name)
+            next_node = node.children[0]
+            return next_node.typ.get_value(next_node, appstruct[index], rest)
+        return appstruct[int(path)]
+
 Seq = Sequence
 
 class String(SchemaType):
@@ -746,7 +860,7 @@ class String(SchemaType):
          object using the encoding and returned.
 
        - A Unicode input value to ``deserialize`` is returned
-         untouched. 
+         untouched.
 
        - A non-Unicode input value to ``deserialize`` is converted to
          a ``str`` object using ``str(value``).  The resulting str
@@ -765,11 +879,11 @@ class String(SchemaType):
     """
     def __init__(self, encoding=None):
         self.encoding = encoding
-    
+
     def serialize(self, node, appstruct):
         if appstruct is null:
             return null
-            
+
         try:
             if isinstance(appstruct, unicode):
                 if self.encoding:
@@ -791,7 +905,7 @@ class String(SchemaType):
     def deserialize(self, node, cstruct):
         if not cstruct:
             return null
-        
+
         try:
             result = cstruct
             if not isinstance(result, unicode):
@@ -827,7 +941,7 @@ class Number(SchemaType):
     def deserialize(self, node, cstruct):
         if cstruct != 0 and not cstruct:
             return null
-        
+
         try:
             return self.num(cstruct)
         except Exception:
@@ -847,7 +961,7 @@ class Integer(Number):
     this type are ignored.
     """
     num = int
-    
+
 Int = Integer
 
 class Float(Number):
@@ -893,7 +1007,7 @@ class Boolean(SchemaType):
     The subnodes of the :class:`colander.SchemaNode` that wraps
     this type are ignored.
     """
-    
+
     def serialize(self, node, appstruct):
         if appstruct is null:
             return null
@@ -903,7 +1017,7 @@ class Boolean(SchemaType):
     def deserialize(self, node, cstruct):
         if cstruct is null:
             return null
-        
+
         try:
             result = str(cstruct)
         except:
@@ -1031,7 +1145,7 @@ class GlobalObject(SchemaType):
     def deserialize(self, node, cstruct):
         if not cstruct:
             return null
-        
+
         if not isinstance(cstruct, basestring):
             raise Invalid(node,
                           _('"${val}" is not a string',
@@ -1096,7 +1210,7 @@ class DateTime(SchemaType):
         if default_tzinfo is _marker:
             default_tzinfo = iso8601.iso8601.Utc()
         self.default_tzinfo = default_tzinfo
-        
+
     def serialize(self, node, appstruct):
         if appstruct is null:
             return null
@@ -1117,7 +1231,7 @@ class DateTime(SchemaType):
     def deserialize(self, node, cstruct):
         if not cstruct:
             return null
-        
+
         try:
             result = iso8601.parse_date(
                 cstruct, default_timezone=self.default_tzinfo)
@@ -1351,14 +1465,14 @@ class SchemaNode(object):
     Arbitrary keyword arguments remaining will be attached to the node
     object unmolested.
     """
-    
+
     _counter = itertools.count()
-    
+
     def __new__(cls, *arg, **kw):
         inst = object.__new__(cls)
         inst._order = cls._counter.next()
         return inst
-        
+
     def __init__(self, typ, *children, **kw):
         self.typ = typ
         self.preparer = kw.pop('preparer', None)
@@ -1422,6 +1536,18 @@ class SchemaNode(object):
     def unflatten(self, fstruct):
         """ Create an appstruct based on the schema represented by
         this node using the fstruct passed. """
+        paths = sorted(fstruct.keys())
+        return self.typ.unflatten(self, paths, fstruct)
+
+    def set_value(self, appstruct, dotted_name, value):
+        """ Uses the schema to set a value in an appstruct from a dotted_name
+        path. """
+        self.typ.set_value(self, appstruct, dotted_name, value)
+
+    def get_value(self, appstruct, dotted_name):
+        """ Traverses the appstruct using the schema and retrieves the value
+        specified by the dotted_name path."""
+        return self.typ.get_value(self, appstruct, dotted_name)
 
     def deserialize(self, cstruct=null):
         """ Deserialize the :term:`cstruct` into an :term:`appstruct` based
@@ -1451,7 +1577,7 @@ class SchemaNode(object):
 
         if self.preparer is not None:
             appstruct = self.preparer(appstruct)
-            
+
         if appstruct is null:
             appstruct = self.missing
             if appstruct is required:
@@ -1514,6 +1640,15 @@ class SchemaNode(object):
                 return node
         raise KeyError(name)
 
+    def __setitem__(self, name, newnode):
+        """ Replace a subnode by name """
+        for idx, node in enumerate(self.children[:]):
+            if node.name == name:
+                self.children[idx] = newnode
+                newnode.name = name
+                return node
+        raise KeyError(name)
+
     def __iter__(self):
         """ Iterate over the children nodes of this schema node """
         return iter(self.children)
@@ -1599,3 +1734,45 @@ class deferred(object):
     def __call__(self, node, kw):
         return self.wrapped(node, kw)
 
+def _unflatten_mapping(node, paths, fstruct,
+                       get_child=None, rewrite_subpath=None):
+    if get_child is None:
+        get_child = node.__getitem__
+    if rewrite_subpath is None:
+        def rewrite_subpath(subpath):
+            return subpath
+    node_name = node.name
+    prefix = node_name + '.'
+    prefix_len = len(prefix)
+    appstruct = {}
+    subfstruct = {}
+    subpaths = []
+    curname = None
+    for path in paths:
+        if path == node_name:
+            # flattened structs contain non-leaf nodes which are ignored
+            # during unflattening.
+            continue
+        assert path.startswith(prefix), "Bad node: %s" % path
+        subpath = path[prefix_len:]
+        if '.' in subpath:
+            name = subpath[:subpath.index('.')]
+        else:
+            name = subpath
+        if curname is None:
+            curname = name
+        elif name != curname:
+            subnode = get_child(curname)
+            appstruct[curname] = subnode.typ.unflatten(
+                subnode, subpaths, subfstruct)
+            subfstruct = {}
+            subpaths = []
+            curname = name
+        subpath = rewrite_subpath(subpath)
+        subfstruct[subpath] = fstruct[path]
+        subpaths.append(subpath)
+    if curname is not None:
+        subnode = get_child(curname)
+        appstruct[curname] = subnode.typ.unflatten(
+            subnode, subpaths, subfstruct)
+    return appstruct
